/* @flow */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.defineSchema = defineSchema;
exports.attribute = attribute;
exports.relationship = relationship;

function defineSchema(attributes) {
  var keys = Object.keys(attributes);

  return function (resource, follow) {
    return keys.reduce(function (result, key) {
      var getter = attributes[key];

      result[key] = getter(resource, key, follow);

      return result;
    }, {});
  };
}

function attribute(resource, key, follow) {
  if (key === 'id' || key === 'type') {
    return resource[key];
  } else if (resource.attributes && key in resource.attributes) {
    return resource.attributes[key];
  } else if (resource.relationships && key in resource.relationships) {
    throw new Error('Key "' + key + '" is relationship but not attribute');
  } else {
    throw new Error('No attribute "' + key + '" found in "' + JSON.stringify(resource) + '"');
  }
}

function relationship(schema) {
  return function (resource, key, follow) {
    if (!resource.relationships || !resource.relationships[key]) {
      throw new Error('No relationship "' + key + '" found in "' + JSON.stringify(resource) + '"');
    }

    return follow(resource.relationships[key].data, schema);
  };
}